<?php
/**
 * ------------------------------------------------------------------------
 * JA Filter Plugin - Content
 * ------------------------------------------------------------------------
 * Copyright (C) 2004-2016 J.O.O.M Solutions Co., Ltd. All Rights Reserved.
 * @license - GNU/GPL, http://www.gnu.org/licenses/gpl.html
 * Author: J.O.O.M Solutions Co., Ltd
 * Websites: http://www.joomlart.com - http://www.joomlancers.com
 * This file may not be redistributed in whole or significant part.
 * ------------------------------------------------------------------------
 */
defined('_JEXEC') or die;

use Joomla\Registry\Registry;
use Joomla\String\StringHelper;
use Joomla\Utilities\ArrayHelper;
JLoader::register('BaseFilterHelper', JPATH_ADMINISTRATOR .'/components/com_jamegafilter/base.php');

class ContentFilterHelper extends BaseFilterHelper {
	function __construct($params = array())
	{
		$this->_db = JFactory::getDbo();
		return parent::__construct($params);
	}
	
	function getLangSuffix()
	{
		$langs = JFactory::getLanguage()->getKnownLanguages();
		$lang_sfx = array();
		foreach ($langs as $lang) {
			$lang_sfx[] = $lang['tag'];
		}
		
		return $lang_sfx;
	}
	
	function getFilterItems($catid)
	{
		$filterItems = array();
		$lang_sfx = $this->getLangSuffix();

		foreach ($lang_sfx AS $lang) {
			$filterItems[strtolower(str_replace('-','_',$lang))] = $this->getItemList($catid,$lang);
		}

		return $filterItems;
	}
	
	public function getItemList($catid, $lang)
	{
		$itemList = new stdCLass();
		$catsList = array($catid);
		$categories = $this->getChildCategories($catid, $lang);
		foreach ($categories as $category) {
			if (in_array($category->language, array('*', $lang))) {
				$catsList[] = $category->id;
			}
		}
		
		$itemIdList = $this->getListId($catsList, $lang);
		if ($itemIdList) {
			foreach ($itemIdList as $id) {
				$property = 'item_'.$id;
				$item = $this->getItem($id, $catsList, $lang);
				if( !empty($item))
					$itemList->{ $property } = $item;
				else
					continue;
			}
		}
		
		return $itemList;
	}
	
	public function getListId($catids, $lang)
	{
		$query = $this->_db->getQuery(true);
		$query->select('id')
				->from('#__content')
				->where('state = 1 AND catid IN (' .implode(',', $catids) . ') AND language IN ("*", "'.$lang.'")' )
				->order('id desc');
		$this->_db->setQuery($query);
		$listId = $this->_db->loadColumn();

		return $listId;
	}
	
	public function getItem($id, $catsList, $lang)
	{
		$app = JFactory::getApplication();
		$baseItem = $this->getBaseItem($id);
		$images = $baseItem->images;
		
		$item = new stdCLass();
		if (in_array($baseItem->language, array('*', $lang))) {
			$item->id = $id;
			$item->lang = $lang;
			$item->hits = (int) $baseItem->hits;
			$item->name = $baseItem->title;
			$item->thumbnail = !empty($images['image_intro']) ? $images['image_intro'] : '';
			$juri = JUri::getInstance();
			if (preg_match('/^\/\//', $item->thumbnail)) {
				$item->thumbnail = $juri->getScheme().':'.$item->thumbnail;
			}

			if ($this->checkDisplayOnFO('desc')) {
				$text = trim($baseItem->introtext . $baseItem->fulltext);
				$item->desc = $text ? $this->getDesc($text) : '';
			}
			
			//Ratings
			if ($this->checkPublished('rating')) {
				$item->rating = $this->getRating($id) ? $this->getRating($id) : 0;
				$item->width_rating = $item->rating * 20;
			}
			
			//Item link
			$item->catid 	= $baseItem->catid;	
			$item->slug        = $baseItem->alias ? ($baseItem->id . ':' . $baseItem->alias) : $baseItem->id;
			$item->catslug     = isset($baseItem->category_alias) ? ($baseItem->catid . ':' . $baseItem->category_alias) : $baseItem->catid;
			
			$item->attr = array();
			if ($this->checkDisplayOnFO('name')) {
				$item->attr['name']['frontend_value'] = $item->name;
				$fieldconfig = $this->getFieldConfig('name');
				$item->attr['name']['title'] = array($fieldconfig['title']);
				$item->attr['name']['type'] = $fieldconfig['type'];
			}

			if ($this->checkDisplayOnFO('hits')) {
				$item->attr['hits']['frontend_value'] = $item->hits;
				$fieldconfig = $this->getFieldConfig('hits');
				$item->attr['hits']['title'] = array($fieldconfig['title']);
				$item->attr['hits']['type'] = $fieldconfig['type'];
			}

			if ($this->checkDisplayOnFO('rating')) {
				$item->attr['rating']['frontend_value'] = $item->width_rating;
				$fieldconfig = $this->getFieldConfig('rating');
				$item->attr['rating']['title'] = array($fieldconfig['title']);
				$item->attr['rating']['type'] = $fieldconfig['type'];
			}

			if ($baseItem->created != '0000-00-00 00:00:00') {
				if ($this->checkPublished('created_date')) {
					$item->created_date = array( strtotime($baseItem->created) );
				}

				if ($this->checkDisplayOnFO('created_date')) {
					$item->attr['created_date']['frontend_value'] = array( strtotime($baseItem->created) );
					$fieldconfig = $this->getFieldConfig('created_date');
					$item->attr['created_date']['title'] = array($fieldconfig['title']);
					$item->attr['created_date']['type'] = $fieldconfig['type'];
				}
			}

			if ($baseItem->modified != '0000-00-00 00:00:00') {
				if ($this->checkPublished('modified_date')) {
					$item->modified_date = array( strtotime($baseItem->modified) );
				}

				if ($this->checkDisplayOnFO('modified_date')) {
					$item->attr['modified_date']['frontend_value'] = array( strtotime($baseItem->modified) );
					$fieldconfig = $this->getFieldConfig('modified_date');
					$item->attr['modified_date']['title'] = array($fieldconfig['title']);
					$item->attr['modified_date']['type'] = $fieldconfig['type'];
				}
			}
			
			if ($baseItem->publish_up != '0000-00-00 00:00:00') {
				if ($this->checkPublished('published_date')) {
					$item->published_date = array( strtotime($baseItem->publish_up) );
				}

				if ($this->checkDisplayOnFO('published_date')) {
					$item->attr['published_date']['frontend_value'] = array( strtotime($baseItem->publish_up) );
					$fieldconfig = $this->getFieldConfig('published_date');
					$item->attr['published_date']['title'] = array($fieldconfig['title']);
					$item->attr['published_date']['type'] = $fieldconfig['type'];
				}
			}
			
			//Attributes
			$this->attr = array();
			$this->getAuthorInfo($baseItem);

			//Category Info
			$this->getCategoryInfo($id, $catsList);
			//Tag Info
			if (!empty($baseItem->tags->tags))
				$this->getTagInfo($baseItem->tags->tags, $lang);
			//Custom fields
			$this->getCustomFieldsInfo($id, $lang);

			$item->attr = array_merge($item->attr, $this->attr)	;
		}
		return $item;
	}

	public function getDesc($desc) {
		$length = 20;
		$desc = strip_tags($desc);
		$exp = explode(' ', $desc);
		$result = '';
		foreach ($exp as $key => $value) {
			if ($key > $length) {
				break;
			}
			$result .= $value . ' ';
		}
		return $result;
	}
	
	public function getRating($itemId) 
	{
		$query = $this->_db->getQuery(true);
		$query->select('rating_sum, rating_count')->from('#__content_rating')->where('content_id = '.(int)$itemId);
		$this->_db->setQuery($query);
		$rating = $this->_db->loadObject();
		if (!$rating)
			return false;
		return round((int)$rating->rating_sum / (int) $rating->rating_count, 0);
	}
	
	public function getBaseItem($id)
	{
		JModelLegacy::addIncludePath(JPATH_ROOT . '/administrator/components/com_content/models', 'ContentModel');
		$model = JModelLegacy::getInstance('Article', 'ContentModel');
		$baseItem = $model->getItem($id);

		return $baseItem;
	}

	public function getAuthorInfo($baseItem) {
		$data = array();
		if ($baseItem->created_by_alias) {
			$data['value'][] = urlencode($baseItem->created_by_alias);
			$data['frontend_value'][] = $baseItem->created_by_alias;
		} else if ($baseItem->created_by) {
			$query = $this->_db->getQuery(true);
			$query->select('*')
				->from('#__users')
				->where('id = ' . $baseItem->created_by);
			$user = $this->_db->setQuery($query)->loadObject();
			if ($user) {
				$data['value'][] = $baseItem->created_by;
				$data['frontend_value'][] = $user->name;
			}
		}

		if (!$data) {
			return;
		}

		if ($this->checkPublished('attr.author.value') ) {
			$this->attr['author'] = $data;
		}

		if ($this->checkDisplayOnFO('attr.author.value')) {
			$this->attr['author']['frontend_value'] = $data['frontend_value'];
			$fieldconfig = $this->getFieldConfig('attr.author.value');
			$this->attr['author']['title'] = array($fieldconfig['title']);
			$this->attr['author']['type'] = $fieldconfig['type'];
		}
	}
	
	public function getCategoryInfo($article_id, $catsList) {
		if (version_compare(JVERSION, '3.7', '<'))
			return;
		
		$query = $this->_db->getQuery(true);
		$query ->select('c.*')
				->from('#__categories as c')
				->join('LEFT', '#__content as a ON a.catid = c.id')
				->where('a.id = ' . (int) $article_id)
				->where('c.id in (' . implode(',', $catsList) . ')')
				->where('c.published = 1');
		$this->_db->setQuery($query);
		$category = $this->_db->loadObject();

		if ($category) {
			$cdata = array();
			$cdata['value'][] = $category->id;
			$cdata['frontend_value'][] = $this->getCatNameAsTree($category);

			if ($this->checkPublished('attr.cat.value')) {
				$this->attr['cat'] = $cdata;
			}

			if ($this->checkDisplayOnFO('attr.cat.value')) {
				$this->attr['cat']['frontend_value'] = $cdata['frontend_value'];
				$fieldconfig = $this->getFieldConfig('attr.cat.value');
				$this->attr['cat']['title'] = array($fieldconfig['title']);
				$this->attr['cat']['type'] = $fieldconfig['type'];
			}
		}
		
		return $this->attr;
	}

	public function getCatNameAsTree($cat) {
		$query = $this->_db->getQuery(true);
		$query->select('*')
			->from('#__categories')
			->where('id = ' . $cat->parent_id . ' and level > 0');
		$this->_db->setQuery($query);
		$result = $this->_db->loadObject();

		if ($result) {
			$result->title = $result->title . ' &raquo; ' . $cat->title;
			return $this->getCatNameAsTree($result);
		} else {
			return $cat->title;
		}
	}
	
	
	public function getCustomFieldsInfo($itemId, $lang) 
	{
		if (version_compare(JVERSION, '3.7', '<'))
			return;
		
		$query = $this->_db->getQuery(true);
		$query->select('f.id, f.title , fv.value, f.type, f.fieldparams')
				->from('#__fields as f')
				->join('LEFT', '#__fields_values as fv ON fv.field_id = f.id')
				->join('LEFT', '#__content as c ON fv.item_id = c.id')
				->where('c.id = '. (int) $itemId);
		$this->_db->setQuery($query);
		$fields = $this->_db->loadObjectList();
		if ($fields) {
			$fdata = array();
			foreach ($fields as $field) {
				if (empty($field->value)) {
					continue;
				}
				switch ($field->type) {
					case 'text':
					case 'editor' :
					case 'textarea' :
					case 'url' :
						$fdata['ct'.$field->id]['value'] .= $field->value;
						$fdata['ct'.$field->id]['frontend_value'] = $field->value;
						break;
					case 'calendar' :
						$fdata['ct'.$field->id]['value'][] = strtotime($field->value);
						$fdata['ct'.$field->id]['frontend_value'][] = strtotime($field->value);
						break;
					case 'integer' :
						$fdata['ct'.$field->id]['value'][] = $field->value;
						$fdata['ct'.$field->id]['frontend_value'][] = $field->value;
						break;
					case 'checkboxes' :
					case 'radio' :
					case 'list' :
					case 'sql' :
						$fdata['ct'.$field->id]['value'][] = str_replace('+','%20',urlencode($field->value));
						$name = $this->getCustomName($field->id, $field->type, $field->value);
						$fdata['ct'.$field->id]['frontend_value'][] = $name;
						break;
					case 'usergrouplist' :
						$gname = $this->getUserGroupName($field->value);
						if ($gname) {
							$fdata['ct'.$field->id]['value'][] = str_replace('+','%20',urlencode($field->value));
							$fdata['ct'.$field->id]['frontend_value'][] = $gname;
						}
						break;
					case 'user' :
						if (JFactory::getUser($field->value)->id) {
							$fdata['ct'.$field->id]['value'][] = str_replace('+','%20',urlencode($field->value));
							$fdata['ct'.$field->id]['frontend_value'][] = JFactory::getUser($field->value)->get('name');
						}
						break;
					case 'imagelist' :
						$fieldparams = json_decode($field->fieldparams);
						$path = 'images/' . $fieldparams->directory . '/';
						$fdata['ct'.$field->id]['value'][] = str_replace('+','%20',urlencode($path . $field->value));
						$fdata['ct'.$field->id]['frontend_value'][] = $path . $field->value;
						break;
					case 'media':
						$fieldparams = json_decode($field->fieldparams);
						$fdata['ct'.$field->id]['value'][] = str_replace('+','%20',urlencode($field->value));
						$fdata['ct'.$field->id]['frontend_value'][] = $field->value;
						break;
					default :
						$fdata['ct'.$field->id]['value'][] = str_replace('+','%20',urlencode($field->value));
						$fdata['ct'.$field->id]['frontend_value'][] = $field->value;
						break;
				}
			}

			foreach ($fdata as $k => $f) {
				if ($this->checkPublished('attr.'.$k.'.value')) {
					$this->attr[$k] = $f;
				}
				
				if ($this->checkDisplayOnFO('attr.'.$k.'.value')) {
					$this->attr[$k]['frontend_value'] = $f['frontend_value'];
					$fieldconfig = $this->getFieldConfig('attr.'.$k.'.value');
					$this->attr[$k]['title'] = array($fieldconfig['title']);
					$this->attr[$k]['type'] = $fieldconfig['type'];
				}
				
			}
		}

		return $this->attr;
	}
	
	public function getUserGroupName($groupId)
	{
		$query = $this->_db->getQuery(true);
		$query->select('title')->from('#__usergroups')->where('id = '. (int)$groupId);
		$this->_db->setQuery($query);
		
		return $this->_db->loadResult();
	}

	public function getTagInfo($tagId, $lang) {
		$query = $this->_db->getQuery(true);
		$query->select('id, title, parent_id')
				->from('#__tags')
				->where('id IN ('.$tagId. ') AND `language` IN ("*", "'.$lang.'") AND published = 1');
		$this->_db->setQuery($query);
		$tags = $this->_db->loadObjectList();
		
		if ($tags) {
			$tdata = array();
			foreach ($tags as $tag) {
				$tdata['value'][] = $tag->id;
				$tdata['frontend_value'][] = $this->getTagTreeName($tag);
			}
			
			if ($this->checkPublished('attr.tag.value')) {
				$this->attr['tag'] = $tdata;
			}
			
			if ($this->checkDisplayOnFO('attr.tag.value')) {
				$this->attr['tag']['frontend_value'] = $tdata['frontend_value'];
				$fieldconfig = $this->getFieldConfig('attr.tag.value');
				$this->attr['tag']['title'] = array($fieldconfig['title']);
				$this->attr['tag']['type'] = $fieldconfig['type'];
			}
		}
		return $this->attr;
	}

	function getTagTreeName($tag) {
		$q = 'SELECT * FROM `#__tags` WHERE id = ' . $tag->parent_id . ' AND id > 1';
		$db = JFactory::getDbo()->setQuery($q);
		$result = $db->loadObject();
		if ($result) {
			$result->title = $result->title . ' &raquo; ' . $tag->title;
			return $this->getTagTreeName($result);
		} else {
			return $tag->title;
		}
	}
	
	public function getCustomFields() 
	{
		if (version_compare(JVERSION, '3.7', '<'))
			return;
		
		$query = $this->_db->getQuery(true);
		$query->select('*')->from('#__fields')->where('context = "com_content.article" AND state = 1');
		$this->_db->setQuery($query);
		$fields = $this->_db->loadObjectList();
		
		return $fields;
	}
	
	public function getCustomName($field_id, $field_type, $field_value) 
	{
		$query = $this->_db->getQuery(true);
		$query->select('fieldparams')
				->from('#__fields')
				->where('id = '. (int) $field_id);
		$this->_db->setQuery($query);
		$fparams = $this->_db->loadResult();
		if ($fparams) {
			$registry = new Registry;
			$registry->loadString($fparams);
			$fparams = $registry->toArray();
			switch ($field_type) {
				case 'sql' :
					$q = $fparams['query'];
					if (!empty($q)) {
						$this->_db->setQuery($q);
						$results = $this->_db->loadObjectList();
						if ($results) {
							foreach ($results as $r) {
								if ($r->value == $field_value)
									return $r->text;
							}
						}
					}
					break;
				default :
					if (!empty($fparams['options'])) {
						foreach ($fparams['options'] as $option) {
							if ($option['value'] == $field_value) {
								return $option['name'];
							}
						}
					}
					break;
			}
		}
	}
	
	// cloned to contentcategory.php. If change need to change there too.
	public function getChildCategories($catid, $lang) 
	{
		$query = $this->_db->getQuery(true);
		$query->select('*')
				->from('#__categories')
				->where('parent_id = '. (int)$catid . ' AND extension IN ("com_content", "system") AND access = 1 AND language IN ("*", "'.$lang.'") AND published = 1');
		$this->_db->setQuery($query);
		
		$childrens = $this->_db->loadObjectList();
		$catids = array();
		if ($childrens) {
			foreach ($childrens as $child) {
				$catids[] = $child;
				if ($this->getChildCategories($child->id, $lang)) {
					foreach ($this->getChildCategories($child->id, $lang) as $c) {
						$catids[] = $c;
					}
				}
			}
		}
		
		return $catids;
	}
	
}
